@@ -32,23 +32,25 @@ public:
 
   virtual bool init(const Glib::ustring& document_uri = Glib::ustring()); //override
 
-  virtual void statusbar_set_text(const Glib::ustring& strText);
-  virtual void statusbar_clear();
+  //virtual void statusbar_set_text(const Glib::ustring& strText);
+  //virtual void statusbar_clear();
 
   /// Get the UIManager so we can merge new menus in.
   Glib::RefPtr<Gtk::UIManager> get_ui_manager();
 
   /** Changes the mode to Data mode, as if the user had selected the Data Mode menu item.
    */
-  virtual void set_mode_data();
-  
-  virtual void add_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
-  virtual void remove_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
-  
-  virtual AppState::userlevels get_userlevel() const;
-  
-  virtual void update_userlevel_ui();
-    
+  void set_mode_data();
+  void set_mode_find();
+
+  void add_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
+  void remove_developer_action(const Glib::RefPtr<Gtk::Action>& refAction);
+
+  AppState::userlevels get_userlevel() const;
+
+  void update_userlevel_ui();
+  void fill_menu_reports(const Glib::ustring& table_name);
+
 protected:
   virtual void init_layout(); //override.
   virtual void init_menus_file(); //override.
@@ -58,6 +60,8 @@ protected:
   virtual void init_create_document(); //override
   virtual bool on_document_load(); //override.
 
+  void fill_menu_tables();
+
   virtual bool offer_new_or_existing();
 
   virtual void on_menu_help_contents();
@@ -78,12 +82,16 @@ protected:
 
   typedef std::list< Glib::RefPtr<Gtk::Action> > type_listActions; 
   type_listActions m_listDeveloperActions; //Only enabled when in developer mode.
-  Glib::RefPtr<Gtk::Action> m_action_mode_data;
+  Glib::RefPtr<Gtk::Action> m_action_mode_data, m_action_mode_find;
   Glib::RefPtr<Gtk::RadioAction> m_action_menu_userlevel_developer, m_action_menu_userlevel_operator;
-  
+
   Gtk::VBox* m_pBoxTop;
   Frame_Glom* m_pFrame;
   Gtk::Label* m_pStatus;
+
+  Glib::RefPtr<Gtk::ActionGroup> m_refNavTablesActionGroup, m_refNavReportsActionGroup;
+  type_listActions m_listNavTableActions, m_listNavReportActions;
+  Gtk::UIManager::ui_merge_id m_menu_tables_ui_merge_id, m_menu_reports_ui_merge_id;
 };
 
 #endif //HEADER_APP_GLOM
